;******************************************************************************
; AUTOHOTKEY SCRIPT FOR USING ORTEK VRC-1100 REMOTE CONTROL WITH MEDIAPORTAL
; --------------------------------------------------------------------------
;
; Author:   CyberSimian (contact via fora at: www.team-mediaportal.com)
; Version:  2016-05-19
;
; o  The Ortek remote control is sold under the following brand names:
;
;    -  Adesso ARC-1100
;    -  Auvisio VRC-1100
;    -  Gmyle
;    -  Hama 00052451
;    -  Ortek VRC-1100
;    -  Sanoxy VRC-1100
;
; o  This script remaps several of the buttons on the Ortek remote control to
;    give improved functionality. The following buttons are changed:
;
;    BROWSER_HOME
;    This button normally invokes the default web browser and displays the
;    user's home page.  To perform this function, Windows intercepts the
;    button before it is received by Centarea HID.  To allow this button to
;    be redefined by the Centarea HID, this script changes BROWSER_HOME to
;    CTRL_F1.
;      If the original BROWSER_HOME function is required, place a semicolon
;    character at the start of the definition lines (this turns the lines
;    into comments).
;
;    POWER
;    This button normally puts the computer into a low-power state, or wakes
;    the computer from the low-power state.  To perform this function,
;    Windows intercepts the button before it is received by AutoHotKey or
;    Centarea HID.  As a result, neither AutoHotKey nor Centarea HID can be
;    used to redefine the button.
;      To specify the action of the POWER button, modify the setting for the
;    "Sleep" button in the "Power buttons and lid" section of the active power
;    plan.
;
;    FAST_FORWARD and FAST_BACKWARD
;    In the "TV Guide" and "Radio Guide" panels, the Centarea HID defines
;    these buttons to scroll the EPG horizontally by three hours.  However,
;    there is a bug somewhere in MediaPortal that afflicts all remote controls,
;    whichever HID is being used (Generic HID, Microsoft MCE, or Centarea HID).
;    The bug is this:
;    -  If the video thumbnail on the EPG panel is showing live TV, recorded
;       TV, or some other video, pressing the FAST_FORWARD or FAST_BACKWARD
;       button scrolls the EPG as expected, but ALSO starts fast-forwarding
;       or fast-rewinding the video in the thumbnail.  However, if the F5 or
;       F6 key on the keyboard is used to scroll the EPG, the thumbnail is
;       not affected.
;    The definitions in this script circumvent this problem by changing the
;    Ortek's FAST_FORWARD and FAST_BACKWARD buttons into F6 and F5, but only
;    in the "TV Guide" and "Radio Guide" panels.
;
;    LEFT_MOUSE
;    This button is changed so that it toggles the MediaPortal Context menu
;    (keyboard shortcut "F9").  This change occurs only when the mouse
;    pointer is within the client area of a MediaPortal window.
;
;    RIGHT_MOUSE
;    This button is changed so that it toggles the MediaPortal OSD (keyboard
;    shortcut "y").  This change occurs only when the mouse pointer is within
;    the client area of a MediaPortal window.
;      Note: if you have specified a timeout for the OSD (typically a few
;    seconds), the right mouse button toggles the OSD on, but does not toggle
;    it off (you must wait for the timeout to expire).
;
;    INFO ("I")
;    This button is a duplicate of the RIGHT_MOUSE button: the Ortek
;    transmits the same signal for both.  As a result, it is not possible to
;    define different actions for these buttons -- INFO will always perform
;    the same action as RIGHT_MOUSE.
;
;    START
;    This button is changed as follows:
;    -  If MediaPortal is not running, it is started and given the focus.
;       This allows MediaPortal to be started from the desktop.  The "MPTray"
;       application that is usually used to start MediaPortal from the
;       desktop does not work with the Ortek remote control, so this script
;       is a replacement for that.
;         Very occasionally MediaPortal does not terminate completely (some
;       part of it remains loaded), and this prevents MediaPortal restarting.
;       To bypass this problem, this script terminates the MediaPortal process
;       if the process is found without any windows.
;    -  If MediaPortal is running in a window but does not have the focus,
;       the MediaPortal window is given the focus.
;    -  If MediaPortal is running but has been minimised to the "System Tray",
;       the MediaPortal window is restored to its previous size and given the
;       focus.
;    -  If MediaPortal is running and has the focus, the MediaPortal "Home"
;       panel (or the alternative "Home" panel) is displayed.
;
; o  To use this script:
;    -  Download AutoHotKey and install it.  AutoHotKey can be found here:
;         https://www.autohotkey.com/
;       The author uses AutoHotKey version 1.0.48.05 on 32-bit Windows Vista.
;       This has proved entirely satisfactory.  However, more-recent versions
;       of AutoHotKey are now available.
;    -  In the script below, modify the path that specifies the location of
;       "MediaPortal.exe" so that it corresponds to the location on your
;       system.
;    -  Use "Windows Explorer" to navigate to this script and then double-
;       click it (or single-click if you have defined your system that way).
;       The script should now be running.  A green icon with an "H" symbol
;       should appear in the "System Tray".
;    -  To stop using the script, right-click the green "H" icon in the
;       "System Tray", and left-click "Exit".
;    -  If you decide that you want to use this script permanently, copy the
;       script into your "Startup" folder, where it will autostart whenever
;       you boot your system.
;
; o  AutoHotKey scripts use the following notation:
;      ; == comment line
;      ! == ALT key
;      ^ == CTRL key
;      + == SHIFT key
;      # == WINDOWS key
;
;
;******************************************************************************
; INITIALISATION (AUTO-EXECUTE SECTION; EXECUTED ONCE-ONLY)
;
; Window-title match mode (1=match start, 2=match substring, 3=match exactly)
  SetTitleMatchMode, 1
;
; Specify the windows where mapping is to occur.
; Mapping occurs in windows that have "MediaPortal" at start of name.
  GroupAdd, mpwin, MediaPortal
;
; Detect hidden windows, so that the START button can restore the MediaPortal
; window when the window has been minimised to the "System Tray".
  DetectHiddenWindows, On
;
; Always get mouse position in screen coordinates
  CoordMode, Mouse, Screen
;
  Return
;
;
;******************************************************************************
; MAPPINGS FOR WHEN MEDIAPORTAL IS NOT RUNNING
;
  #IfWinNotExist, ahk_group mpwin
;
; START button
  #!enter::
;-- Check if MediaPortal did not stop cleanly when closed previously;
;-- this happens occasionally, and prevents MediaPortal restarting.
    Process, Exist, MediaPortal.exe
    pid := ErrorLevel
    if (pid != 0)
    {
;---- MediaPortal did not close cleanly -- terminate it forcibly.
      Process, Close, MediaPortal.exe
;---- Wait for command to complete (value is milliseconds).
      Sleep, 500
    }
;-- Start MediaPortal.
;-- De-comment the line appropriate for your Windows installation.
;-- For 32-bit Windows:
    Run, "C:\Program Files\Team MediaPortal\MediaPortal\MediaPortal.exe"
;-- For 64-bit Windows:
;   Run, "C:\Program Files (x86)\Team MediaPortal\MediaPortal\MediaPortal.exe"
    Return
;
;
;******************************************************************************
; MAPPINGS FOR WHEN MEDIAPORTAL IS RUNNING BUT DOES NOT HAVE THE FOCUS
; MAPPINGS FOR WHEN MEDIAPORTAL IS MINIMISED TO THE SYSTEM TRAY
;
  #IfWinNotActive, ahk_group mpwin
;
; START button
  #!enter::
    WinActivate, ahk_group mpwin
    WinShow, ahk_group mpwin
    Return
;
;
;******************************************************************************
; MAPPINGS FOR WHEN MEDIAPORTAL IS RUNNING AND DOES HAVE THE FOCUS
;
  #IfWinActive, ahk_group mpwin
;
; BROWSER_HOME button
; The following definition changes BROWSER_HOME to CTRL_F1.
  browser_home::
    SendInput, {ctrl down}{f1 down}{f1 up}{ctrl up}
    Return
;
; POWER button
; This button cannot be trapped by AutoHotKey.
; sleep::
;   Return
;
; FAST_BACKWARD button
  ^+b::
    WinGetActiveTitle, title
    If (title = "MediaPortal - TV/Guide")
    Or (title = "MediaPortal - Radio guide")
      SendInput, {ctrl up}{shift up}{f5 down}{f5 up}{shift down}{ctrl down}
    Else
      SendInput, {ctrl down}{shift down}{b down}{b up}{shift up}{ctrl up}
    Return
;
; FAST_FORWARD button
  ^+f::
    WinGetActiveTitle, title
    If (title = "MediaPortal - TV/Guide")
    Or (title = "MediaPortal - Radio guide")
      SendInput, {ctrl up}{shift up}{f6 down}{f6 up}{shift down}{ctrl down}
    Else
      SendInput, {ctrl down}{shift down}{f down}{f up}{shift up}{ctrl up}
    Return
;
; LEFT_MOUSE button
  lbutton::
    If (MouseInMpWindow() = 1)
      SendInput, {f9}
    Else
    {
      SendInput, {lbutton down}
;---- Use "KeyWait" so that button can be held down to drag and resize windows.
      KeyWait, lbutton
      SendInput, {lbutton up}
    }
    Return
;
; RIGHT_MOUSE button
  rbutton::
    If (MouseInMpWindow() = 1)
      SendInput, {y}
    Else
    {
      SendInput, {rbutton down}
;---- Use "KeyWait" so that button can be held down.
      KeyWait, rbutton
      SendInput, {rbutton up}
    }
    Return
;
; START button
; By NOT mapping the START button here, the action of the button when
; MediaPortal has the focus can be specified in "Centarea HID.xml".
; The START button generates the following sequence of keystrokes:
; -  press and hold WINDOWS key
; -  press and hold ALT key
; -  (press RETURN key)  (not detected)
; -  release ALT key
; -  release WINDOWS key
; -  release RETURN key
; #!enter::
;   SendInput, {lwin up}{alt up}{h down}{h up}{alt down}{lwin down}
;   Return
;
;
;******************************************************************************
; SUBROUTINES

MouseInMpWindow()
{
; Get mouse position and id of window containing mouse pointer.
  MouseGetPos, mouse_x, mouse_y, id,
; Check if mouse is within window's client area.
  If (InClient(id,mouse_x,mouse_y) = 0)
    Return, (0)
; Get title of window containing mouse pointer.
  WinGetTitle, title, ahk_id %id%
; Check for non-MP window.
  If (InStr(title,"MediaPortal") = 0)
    Return, (0)
; Check for "MediaPortal - Configuration" window.
; Check for "MediaPortal - TV Server Configuration" window.
  If (InStr(title,"Configuration") != 0)
    Return, (0)
; Check for "MediaPortal - Setup" window (remote control mappings)
  If (InStr(title,"Setup") != 0)
    Return, (0)
; Get window text.
  WinGetText, text, ahk_id %id%
; Check for warning window issued when MP Config starts with MP still running.
  If (InStr(text,"MediaPortal has to be closed for configuration") != 0)
    Return, (0)
; Window is a genuine MP window.
  Return, (1)
}

InClient(hwnd, x, y)
{
; get size of client area (width and height)
  GetClientRect(hwnd, client_w, client_h)
; get screen coords of top left corner of client area
  ClientToScreen(hwnd, 0, 0, client_x, client_y)
; check specified position against limits of client area
  in_client := 1
  If (x < client_x)
  Or (x >= client_x + client_w)
  Or (y < client_y)
  Or (y >= client_y + client_h)
    in_client := 0
  Return, (in_client)
}

GetClientRect(hwnd, ByRef client_w, ByRef client_h)
{
  VarSetCapacity(rect, 16)
  DllCall("GetClientRect", "uint", hwnd, "uint", &rect)
  client_w := NumGet(rect, 8, "int")
  client_h := NumGet(rect, 12, "int")
  Return
}

ClientToScreen(hwnd, client_x, client_y, ByRef screen_x, ByRef screen_y)
{
  VarSetCapacity(point, 8)
  NumPut(client_x, point, 0, "int")
  NumPut(client_y, point, 4, "int")
  DllCall("ClientToScreen", "uint", hwnd, "uint", &point)
  screen_x := NumGet(point, 0, "int")
  screen_y := NumGet(point, 4, "int")
  Return
}

